#!/usr/bin/perl

# This script analyzes Samba's log and outputs file access logs as CSV
# The format is Timestamp , NETBIOS_NAME , USER_NAME , FILE_NAME , ERROR(if error occurs)

# This script run correctly with following Samba config
# in [Globa] section
# debug uid = true
# log level = 3
# max log size = 0

# Copyright (C) 2004 Ryo SHIMIZU (furyo@on-air.ne.jp,http://furyo.on-air.ne.jp)

use strict;

# You should edit following 3 lines. $path_to_samba_log and $log_output_dir must be finished ' / ' (scamped work)
my $path_to_samba_log = '/var/log/samba/';
my $log_output_dir = '/var/log/samba/';
my $log_temp = '/tmp/smbtemp';


my $date;
my @temp = &yyyymmdd();

# daily log is outputed to '/log/top/yyyy/mm/dd'
$date = join "" , @temp;
foreach(@temp)
{
	$log_output_dir .= "$_/";
}
&make_dir($log_output_dir);
undef @temp;

# recognize log.* as log file(Samba default ; log file = log.%m)
opendir DIR , $path_to_samba_log or die "cannot read Samba log $!";
my @log_file = grep { /log\..*/ } readdir DIR;
closedir DIR;

foreach my $file(@log_file)	
{
	print "checking $file ..." , "\n";
	my $netbios_name = &get_machine_name($file);
	$file = $path_to_samba_log . $file;
	
	my $here = '';
	my $prev = '';

	open FILE , $file;

	my $user_name;
	my $file_name;
	my $read_flag;
	my $write_flag;
	my $error_msg = '';

	my @temp;
	my $now_date = '';

	open LOG_OUT , ">$log_temp" or die "cannot open $log_temp";

	while($here = <FILE>)
	{
		if($here =~ /unix_clean_name \[(.*?)\]/)
		{
			$file_name = $1;
			next if $file_name =~ /^\//;

			#ignore files
			next if $file_name eq '';
			next if $file_name eq '.';
			next if $file_name eq './';
			next if $file_name =~ /\*/;

			#before filename , log includes UID
			$user_name = &get_user_name($prev);
			@temp = &gettime($prev);
			$now_date = join '' , @temp;
			$here = <FILE>;

			my $ts = join('' , &gettime($here));

			my $quoted_filename = quotemeta($file_name);
			while((!(&line_is_timestamp($here))) or $ts eq $now_date)
			{
				$error_msg = '';
				if($here =~ /Error opening file $quoted_filename \((.*?)\)/)
				{
					$error_msg = $1;
					$error_msg = '' if $error_msg eq $quoted_filename;
					last if $error_msg;
				}
				elsif($here =~ /vfs_stat of $quoted_filename failed \((.*?)\)/)
				{
					$error_msg = $1;
					$error_msg = '' if $error_msg eq $quoted_filename;
					last if $error_msg;
				}

				$ts = join('' , &gettime($here));
				$here = <FILE>;
			}
			print LOG_OUT &csvout(&format_time(@temp) , $netbios_name ,  $user_name , $file_name , $error_msg);
		}
		$prev = $here;
	}
	close LOG_OUT;
	close FILE;
	system "sort $log_temp |uniq > $log_output_dir${netbios_name}_$date.csv";
}

unlink $log_temp;

sub gettime	#return (year , month , day , hour , min , sec)
{
	my $line = shift;
	
	my @ret = ();
	if($line =~ /\[(\d+)\/(\d*)\/(\d*) (\d*):(\d*):(\d*),/)
	{
		@ret = ($1 , $2 , $3 , $4 , $5 , $6);
	}
	return @ret;
}

sub format_time
{
	my @date = @_;
	return "'$date[0]/$date[1]/$date[2] $date[3]:$date[4]:$date[5]";
}

sub csvout
{
	my @arg = @_;
	my $ret;
	my $i;
	for($i = 0;$i < $#arg;$i++)
	{
		$ret .= "\"$arg[$i]\",";
	}
	$ret .= "\"$arg[$i]\"\n";
	return $ret;
}

sub get_machine_name
{
	my $log_file = shift;
	my $ret = (split /\./ , $log_file)[1];
	return $ret;
}

sub line_is_timestamp
{
	my $line = shift;
	if($line =~ /^\[/)
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

sub get_user_name
{
	my $line = shift;
	my $ret;
	$line =~ /effective\((\d*),/;
	$ret = $1;
	$ret = getpwuid($ret);
	return $ret;
}

sub yyyymmdd
{
	my @time = localtime();
	my $year;
	my $month;
	my $day;

	$year = sprintf("%04d" , $time[5] + 1900);
	$month = sprintf("%02d" , $time[4] + 1);
	$day = sprintf("%02d" , $time[3]);
	return ($year , $month , $day);
}

sub make_dir
{
	my $dir = shift;
	if(-e $dir)
	{
		return 1;
	}
	else
	{
		system "mkdir -p $dir";
	}
	return 1;
}
